import rch.arrays
import rch.df
import rch.gdf
import rch.hydro
import rch.sudoku
import rch.web

__all__ = ['arrays', 'df', 'gdf', 'hydro', 'sudoku', 'web']
__version__ = '0.13'
__author__ = 'Riley Hales'
__license__ = 'BSD Clear'
